import datetime
import functools
import operator
from collections import Counter

from pyspark import SparkConf
from pyspark.sql import SparkSession
from tool import get_jerry_test, get_tag3_user_log


# [{'激光': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873}, {'手术': 1.949194898204873}]
# {'手术': 5.8475846946146195, '激光': 1.949194898204873}
def merge_values(list_of_dict):
    return dict(functools.reduce(operator.add, map(Counter, list_of_dict)))


# [("a", 1), ("b", 2)]
# {'a': 1, 'b': 2}
# [{'缩阴道激光': 1.949194898204873}, {'': 1.949194898204873}, {'': 1.949194898204873}, {'私处手术': 1.949194898204873}]
def make_dict_from_pair(x):
    try:
        x[0].remove("")
    except Exception as e:
        pass
    return dict(zip(x[0], [x[1]] * len(x[0])))


def update_tag3_user_portrait(cl_id):
    user_df = get_tag3_user_log(cl_id)
    if not user_df.empty:
        user_df["first_solutions"] = list(
            zip(user_df["first_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_solutions"] = list(
            zip(user_df["second_solutions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_demands"] = list(
            zip(user_df["first_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_demands"] = list(
            zip(user_df["second_demands"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["first_positions"] = list(
            zip(user_df["first_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["second_positions"] = list(
            zip(user_df["second_positions"].apply(lambda x: x.split(",")), user_df["tag_score"]))
        user_df["projects"] = list(zip(user_df["projects"].apply(lambda x: x.split(",")), user_df["tag_score"]))

        user_df["first_solutions_dict"] = user_df["first_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_solutions_dict"] = user_df["second_solutions"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_demands_dict"] = user_df["first_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_demands_dict"] = user_df["second_demands"].apply(lambda x: make_dict_from_pair(x))
        user_df["first_positions_dict"] = user_df["first_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["second_positions_dict"] = user_df["second_positions"].apply(lambda x: make_dict_from_pair(x))
        user_df["projects_dict"] = user_df["projects"].apply(lambda x: make_dict_from_pair(x))

        first_solutions_list = user_df["first_solutions_dict"].tolist()
        second_solutions_list = user_df["second_solutions_dict"].tolist()
        first_demands_list = user_df["first_demands_dict"].tolist()
        second_demands_list = user_df["second_demands_dict"].tolist()
        first_positions_list = user_df["first_positions_dict"].tolist()
        second_positions_list = user_df["second_positions_dict"].tolist()
        projects_list = user_df["projects_dict"].tolist()

        res = {
            "first_demands": merge_values(first_demands_list),
            "second_demands": merge_values(second_demands_list),
            "first_solutions": merge_values(first_solutions_list),
            "second_solutions": merge_values(second_solutions_list),
            "first_positions": merge_values(first_positions_list),
            "second_positions": merge_values(second_positions_list),
            "projects": merge_values(projects_list)
        }

        # TODO doris:user_portrait:tag3:device_id:
        # cl_id_portrait_key = "doris:test:device_id:" + str(cl_id)
        # redis_client.set(cl_id_portrait_key, json.dumps(res))
        # redis_client.expire(cl_id_portrait_key, 60*60*24)

        return cl_id


def consume_kafka():
    # TODO 30
    sql = "select distinct cl_id from kafka_tag3_log where log_time > UNIX_TIMESTAMP(DATE_SUB(NOW(), INTERVAL 5 day))"
    db, cursor = get_jerry_test()
    cursor.execute(sql)
    device_ids_lst = [i[0] for i in cursor.fetchall()]
    db.close()
    cursor.close()

    sparkConf = SparkConf().set("spark.hive.mapred.supports.subdirectories", "true") \
                           .set("spark.hadoop.mapreduce.input.fileinputformat.input.dir.recursive", "true") \
                           .set("spark.tispark.plan.allow_index_double_read", "false") \
                           .set("spark.tispark.plan.allow_index_read", "true") \
                           .set("spark.sql.extensions", "org.apache.spark.sql.TiExtensions") \
                           .set("spark.tispark.pd.addresses", "172.16.40.170:2379") \
                           .set("spark.io.compression.codec", "lzf") \
                           .set("spark.driver.maxResultSize", "8g") \
                           .set("spark.sql.avro.compression.codec", "snappy")

    spark = SparkSession.builder.config(conf=sparkConf).enableHiveSupport().getOrCreate()
    spark.sparkContext.setLogLevel("WARN")
    spark.sparkContext.addPyFile("/srv/apps/ffm-baseline_git/eda/smart_rank/tool.py")

    device_ids_lst_rdd = spark.sparkContext.parallelize(device_ids_lst, numSlices=1000)
    result = device_ids_lst_rdd.repartition(100).map(lambda x: update_tag3_user_portrait(x))
    # result.foreach(print)
    result.collect()
    spark.stop()


if __name__ == "__main__":
    # cl_id = "866017030837899"
    # res = update_tag3_user_portrait(cl_id)
    # print(res)

    start = datetime.datetime.now()
    consume_kafka()
    end = datetime.datetime.now()
    print(end - start)
    print("done")
